/**
 *    file  checktools.inc
 *   brief 
 *
 *  test
 *  bug        A remark about a known bug in the code.
 *  version
 *  since
 *  note       A brief remark about the implementation to help clarify.
 *  attention  An important remark that may cause code to break
 *  warning    An import remark that may depend on random conditions etc.
 *  see        A reference to a class or a link to documentation (e.g. http://document.1a.com)
 *
 *  author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  internal
 *       Created:  19/07/2024
 *      Revision:  none
 * Last modified:  2024-07-19 15:28
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <unistd.h>                             /* pid_t getpid() */
#include <sys/stat.h>                           /* mkdir(), chdir(), mode_t */

#ifdef GCOV
#include <signal.h>                             /* signal(), SIGABRT, SIG_DFL */
void __gcov_dump();
static void signals_handler(int sig)
{
    (void)sig;
    signal(SIGABRT,SIG_DFL);
    signal(SIGSEGV,SIG_DFL);
    __gcov_dump();
}
#endif /* GCOV */

static void signals_catch() {
#ifdef GCOV
    signal(SIGABRT,signals_handler);
    signal(SIGSEGV,signals_handler);
#endif
}

static void signals_release() {
#ifdef GCOV
    signal(SIGABRT,SIG_DFL);
    signal(SIGSEGV,SIG_DFL);
#endif
}

static pid_t _forktest_pid;

static void forktest_init() {
    _forktest_pid = getpid();
}

#define forktest_only if (_forktest_pid==getpid()) return

static void forktest_gprofdir() {
#ifdef GPROF
    if (_forktest_pid!=getpid()) {
        /* Forked : mkdir/chdir to a subdir for gmon.out */
        char dirname[20];
        int result = snprintf(dirname, 20, "%d", getpid());
        if ((0<result)&&(20>result)) {
            mkdir("gprof", S_IRWXU);
            chdir("gprof");
            mkdir(dirname, S_IRWXU);
            chdir(dirname);
        } else {
            fprintf(stderr,"Unable to create GPROF/gmon.out subdir\n");
            abort();
        }
    }
#endif /* GPROF */
}

/* vim:se filetype=c : */
