/**   \file  suite_svcmgr-register.c
 *   \brief  Check unit testing framework main function for SvcMgr
 *  \author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  \internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-07-19 15:18
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "svcmgr.h"
#include "oom.h"                                /* OOM simulation */
#include <check.h>
#include <unistd.h>                             /* pid_t getpid() */
#include <sys/stat.h>                           /* mkdir(), chdir(), mode_t */
#include <stdio.h>                              /* fprintf */
#include <stdlib.h>                             /* abort() */
#include "checktools.inc"

static uint8_t svc_func (va_list p_ap)
{
    uint8_t i=0;

    for (i=0; i<3; i++) {
        printf ("arg[%d]=%d, ",i,va_arg(p_ap,int));
    }
    printf("\n");
    return 0;
}

START_TEST(test_1)
{
    svcmgr_register("a",svc_func);
    svcmgr_call("a",1,2,3);
}
END_TEST

START_TEST(test_2)
{
    svcmgr_register("b",svc_func);
    svcmgr_call("a",4,5,6);
    svcmgr_call("b",7,8,9);
}
END_TEST

START_TEST(test_3)
{
    svcmgr_register("a",svc_func);
    svcmgr_call("a",10,11,12);
    svcmgr_call("b",13,14,15);
}
END_TEST

START_TEST(test_4)
{
    svcmgr_register("b/c",svc_func);
    svcmgr_call("a",16,17,18);
    svcmgr_call("b",19,20,21);
    svcmgr_call("b/c",22,23,24);
    svcmgr_call("c",25,26,27);
}
END_TEST

START_TEST(test_5)
{
    svcmgr_unregister("a");
}
END_TEST

/* Begining of test case */
void svcmgr_unchecked_setup()
{
    forktest_init();
}
/* Begining of test (potentially in child) */
void svcmgr_checked_setup()
{
    signals_catch();
    forktest_gprofdir();
}
/* End of test (potentially in child) */
void svcmgr_checked_teardown()
{
    signals_release();
}
/* End of testcase */
void svcmgr_unchecked_teardown()
{
}

Suite* svcmgr_register_suite()
{
    Suite *s;
    TCase *tc;

    s = suite_create("SvcMgr");
    tc = tcase_create("SvcMgr");
    /* tcase_set_tags(tc,"tag1 tag2"); */
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, svcmgr_checked_setup, svcmgr_checked_teardown);
    tcase_add_unchecked_fixture(tc, svcmgr_unchecked_setup, svcmgr_unchecked_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test(tc, test_1);
    tcase_add_test(tc, test_2);
    tcase_add_test(tc, test_3);
    tcase_add_test(tc, test_4);
    tcase_add_test(tc, test_5);

    return s;
}
